=== modified file 'libunity-2d-private/src/unity2ddeclarativeview.cpp'
--- libunity-2d-private/src/unity2ddeclarativeview.cpp	2012-02-09 10:56:45 +0000
+++ libunity-2d-private/src/unity2ddeclarativeview.cpp	2012-02-14 17:19:21 +0000
@@ -46,12 +46,18 @@
 GOBJECT_CALLBACK0(activeWorkspaceChangedCB, "onActiveWorkspaceChanged");
 
 Unity2DDeclarativeView::Unity2DDeclarativeView(QWidget *parent) :
-    QDeclarativeView(parent),
+    QGraphicsView(parent),
     m_screenInfo(NULL),
     m_useOpenGL(false),
     m_transparentBackground(false),
-    m_last_focused_window(None)
+    m_last_focused_window(None),
+    m_rootItem(NULL)
 {
+    setScene(&m_scene);
+    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    setFrameStyle(NoFrame);
+
     if (!QFileInfo(UNITY_2D_SCHEMA_FILE).exists()) {
         m_useOpenGL = false;
     } else {
@@ -68,6 +74,47 @@
 {
 }
 
+QDeclarativeEngine* Unity2DDeclarativeView::engine()
+{
+    static QDeclarativeEngine* engine = new QDeclarativeEngine();
+    return engine;
+}
+
+QDeclarativeContext* Unity2DDeclarativeView::rootContext() const
+{
+    return engine()->rootContext();
+}
+
+QGraphicsObject* Unity2DDeclarativeView::rootObject() const
+{
+    return m_rootItem;
+}
+
+void Unity2DDeclarativeView::setSource(const QUrl &source, const QMap<const char*, QVariant> &rootObjectProperties)
+{
+    QDeclarativeComponent* component = new QDeclarativeComponent(engine(), source, this);
+    QObject *instance = component->beginCreate(rootContext());
+    QMap<const char*, QVariant>::const_iterator it = rootObjectProperties.begin();
+    QMap<const char*, QVariant>::const_iterator itEnd = rootObjectProperties.end();
+    for ( ; it != itEnd; ++it) {
+        instance->setProperty(it.key(), it.value());
+    }
+    component->completeCreate();
+    m_rootItem = qobject_cast<QDeclarativeItem *>(instance);
+    connect(m_rootItem, SIGNAL(widthChanged()), SLOT(resizeToRootObject()));
+    connect(m_rootItem, SIGNAL(heightChanged()), SLOT(resizeToRootObject()));
+    resizeToRootObject();
+    m_scene.addItem(m_rootItem);
+    m_source = source;
+}
+
+void Unity2DDeclarativeView::resizeToRootObject()
+{
+    QSize size(m_rootItem->width(), m_rootItem->height());
+    resize(size);
+    Q_EMIT sceneResized(size);
+}
+
 bool Unity2DDeclarativeView::useOpenGL() const
 {
     return m_useOpenGL;
@@ -90,6 +137,11 @@
     return m_transparentBackground;
 }
 
+QUrl Unity2DDeclarativeView::source() const
+{
+    return m_source;
+}
+
 void Unity2DDeclarativeView::setTransparentBackground(bool transparentBackground)
 {
     if (transparentBackground == m_transparentBackground) {
@@ -164,13 +216,13 @@
 
 void Unity2DDeclarativeView::showEvent(QShowEvent* event)
 {
-    QDeclarativeView::showEvent(event);
+    QGraphicsView::showEvent(event);
     Q_EMIT visibleChanged(true);
 }
 
 void Unity2DDeclarativeView::hideEvent(QHideEvent* event)
 {
-    QDeclarativeView::hideEvent(event);
+    QGraphicsView::hideEvent(event);
     Q_EMIT visibleChanged(false);
 }
 

=== modified file 'libunity-2d-private/src/unity2ddeclarativeview.h'
--- libunity-2d-private/src/unity2ddeclarativeview.h	2012-02-07 11:57:10 +0000
+++ libunity-2d-private/src/unity2ddeclarativeview.h	2012-02-14 17:19:21 +0000
@@ -17,11 +17,14 @@
 #ifndef UNITY2DDECLARATIVEVIEW_H
 #define UNITY2DDECLARATIVEVIEW_H
 
-#include <QDeclarativeView>
+#include <QDeclarativeContext>
+#include <QDeclarativeEngine>
+#include <QDeclarativeItem>
+#include <QGraphicsView>
 
 class ScreenInfo;
 
-class Unity2DDeclarativeView : public QDeclarativeView
+class Unity2DDeclarativeView : public QGraphicsView
 {
     Q_OBJECT
 
@@ -38,12 +41,18 @@
     // getters
     bool useOpenGL() const;
     bool transparentBackground() const;
+    QUrl source() const;
     QPoint globalPosition() const;
     ScreenInfo* screen() const;
 
     // setters
     void setUseOpenGL(bool);
     void setTransparentBackground(bool);
+    void setSource(const QUrl& source, const QMap<const char*, QVariant> &rootObjectProperties = QMap<const char*, QVariant>());
+
+    static QDeclarativeEngine *engine();
+    QDeclarativeContext* rootContext() const;
+    QGraphicsObject* rootObject() const;
 
 Q_SIGNALS:
     void useOpenGLChanged(bool);
@@ -52,6 +61,7 @@
     void screenChanged(ScreenInfo*);
     void visibleChanged(bool);
     void activeWorkspaceChanged();
+    void sceneResized(QSize size);
 
 protected:
     void setupViewport();
@@ -67,6 +77,7 @@
 
 private Q_SLOTS:
     void onActiveWorkspaceChanged();
+    void resizeToRootObject();
 
 private:
     void saveActiveWindow();
@@ -74,7 +85,11 @@
 
     bool m_useOpenGL;
     bool m_transparentBackground;
+    QUrl m_source;
     WId m_last_focused_window;
+
+    QGraphicsScene m_scene;
+    QDeclarativeItem* m_rootItem;
 };
 
 Q_DECLARE_METATYPE(Unity2DDeclarativeView*)

=== modified file 'shell/Shell.qml'
--- shell/Shell.qml	2012-02-14 07:13:39 +0000
+++ shell/Shell.qml	2012-02-14 17:19:21 +0000
@@ -23,6 +23,9 @@
 
 Item {
     id: shell
+
+    property variant declarativeView
+
     /* Space reserved by strutManager is taken off screen.availableGeometry but
        we want the shell to take all the available space, including the one we
        reserved ourselves via strutManager. */

=== modified file 'shell/app/shelldeclarativeview.cpp'
--- shell/app/shelldeclarativeview.cpp	2012-02-14 11:01:49 +0000
+++ shell/app/shelldeclarativeview.cpp	2012-02-14 17:19:21 +0000
@@ -90,22 +90,24 @@
         posToMove.setX(availableGeometry.width() - width());
     }
 
-    QList<StrutManager *> strutManagers = rootObject()->findChildren<StrutManager*>();
-    Q_FOREACH(StrutManager *strutManager, strutManagers) {
-        if (strutManager->enabled()) {
-            // Do not push ourselves
-            switch (strutManager->edge()) {
-                case Unity2dPanel::TopEdge:
-                    posToMove.ry() -= strutManager->realHeight();
-                break;
+    if (rootObject() != NULL) {
+        QList<StrutManager *> strutManagers = rootObject()->findChildren<StrutManager*>();
+        Q_FOREACH(StrutManager *strutManager, strutManagers) {
+            if (strutManager->enabled()) {
+                // Do not push ourselves
+                switch (strutManager->edge()) {
+                    case Unity2dPanel::TopEdge:
+                        posToMove.ry() -= strutManager->realHeight();
+                    break;
 
-                case Unity2dPanel::LeftEdge:
-                    if (qApp->isLeftToRight()) {
-                        posToMove.rx() -= strutManager->realWidth();
-                    } else {
-                        posToMove.rx() += strutManager->realWidth();
-                    }
-                break;
+                    case Unity2dPanel::LeftEdge:
+                        if (qApp->isLeftToRight()) {
+                            posToMove.rx() -= strutManager->realWidth();
+                        } else {
+                            posToMove.rx() += strutManager->realWidth();
+                        }
+                    break;
+                }
             }
         }
     }
@@ -159,7 +161,9 @@
        will remove the flags when the window is hidden */
     setWMFlags();
     if (source().isEmpty()) {
-        setSource(m_sourceFileUrl);
+        QMap<const char*, QVariant> rootObjectProperties;
+        rootObjectProperties.insert("declarativeView", QVariant::fromValue(this));
+        setSource(m_sourceFileUrl, rootObjectProperties);
     }
 }
 

=== modified file 'shell/app/shellmanager.cpp'
--- shell/app/shellmanager.cpp	2012-02-14 12:26:47 +0000
+++ shell/app/shellmanager.cpp	2012-02-14 17:19:21 +0000
@@ -83,9 +83,6 @@
     view->engine()->addImportPath(unity2dImportPath());
     view->engine()->setBaseUrl(QUrl::fromLocalFile(unity2dDirectory() + "/shell/"));
 
-    /* Load the QML UI, focus and show the window */
-    view->setResizeMode(QDeclarativeView::SizeViewToRootObject);
-    view->rootContext()->setContextProperty("declarativeView", view);
     // WARNING This declaration of dashClient used to be in Unity2d/plugin.cpp
     // but it lead to locks when both the shell and the spread were started
     // at the same time since SpreadMonitor QDBusServiceWatcher::serviceRegistered

=== modified file 'spread/app/spread.cpp'
--- spread/app/spread.cpp	2012-02-08 11:05:30 +0000
+++ spread/app/spread.cpp	2012-02-14 17:19:21 +0000
@@ -60,7 +60,6 @@
     view.rootContext()->setContextProperty("control", &control);
 
     /* Load the QML UI, focus and show the window */
-    view.setResizeMode(QDeclarativeView::SizeRootObjectToView);
     view.rootContext()->setContextProperty("declarativeView", &view);
     view.rootContext()->setContextProperty("spreadView", &view);
     view.setSource(QUrl("./Workspaces.qml"));

=== modified file 'spread/app/spreadview.cpp'
--- spread/app/spreadview.cpp	2012-02-07 11:57:10 +0000
+++ spread/app/spreadview.cpp	2012-02-14 17:19:21 +0000
@@ -70,7 +70,7 @@
     this->viewport()->grabKeyboard();
     this->viewport()->grabMouse();
 
-    QDeclarativeView::focusInEvent(event);
+    Unity2DDeclarativeView::focusInEvent(event);
 }
 
 /* This is not strictly necessary as X11 will cancel any grabs when the
@@ -83,7 +83,7 @@
     this->viewport()->releaseMouse();
     this->viewport()->removeEventFilter(this);
 
-    QDeclarativeView::focusOutEvent(event);
+    Unity2DDeclarativeView::focusOutEvent(event);
 }
 
 bool SpreadView::eventFilter(QObject *obj, QEvent *event) {

